/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/DiagnosticTemplate.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import static com.netapp.common.flow.FlowConstants.ABORT_TASKS_ELEMENT;
import static com.netapp.common.flow.FlowConstants.DIAGNOSTIC_ELEMENT;
import static com.netapp.common.flow.FlowConstants.DIAGNOSTIC_OPERATION_NAME;
import static com.netapp.common.flow.FlowConstants.FAILED_OPERATION_CLIENT_REQUEST_ID;
import static com.netapp.common.flow.FlowConstants.FAILED_OPERATION_ID;
import static com.netapp.common.flow.FlowConstants.FAILED_OPERATION_REGISTERS;
import static com.netapp.common.flow.FlowConstants.FAILED_OPERATION_TYPE;
import static com.netapp.common.flow.FlowConstants.REGISTER_ELEMENT;
import static com.netapp.common.flow.FlowConstants.TASKS_ELEMENT;
import static com.netapp.common.flow.FlowConstants.THREADPOOL_ATTRIBUTE;
import static com.netapp.common.flow.FlowConstants.WORKFLOW_SCHEMA_NAMESPACE;

import org.w3c.dom.Node;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.xml.DOMUtils;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DiagnosticTemplate extends OperationTemplate {
    private static final NALogger logger = NALogger.getLogger(DiagnosticTemplate.class);
    
    protected static final RegisterTemplate REGISTER_OPERATION_ID = new RegisterTemplate(FAILED_OPERATION_ID, FAILED_OPERATION_ID, null, "String");
    protected static final RegisterTemplate REGISTER_OPERATION_CLIENT_REQUEST_ID = new RegisterTemplate(FAILED_OPERATION_CLIENT_REQUEST_ID, FAILED_OPERATION_CLIENT_REQUEST_ID, null, "String");
    protected static final RegisterTemplate REGISTER_OPERATION_TYPE = new RegisterTemplate(FAILED_OPERATION_TYPE, FAILED_OPERATION_TYPE, null, "String");
    protected static final RegisterTemplate REGISTER_OPERATION_REGISTERS = new RegisterTemplate(FAILED_OPERATION_REGISTERS, FAILED_OPERATION_REGISTERS, null, "java.util.Map<String, Object>");
    
    DiagnosticTemplate(Node node, TaskInstanceFactory taskInstanceFactory) {
        super(DIAGNOSTIC_OPERATION_NAME, DOMUtils.getAttribute(node, THREADPOOL_ATTRIBUTE), taskInstanceFactory);
        if (!DOMUtils.isElement(node, WORKFLOW_SCHEMA_NAMESPACE, DIAGNOSTIC_ELEMENT)) {
            throw new FlowException(logger.error(MsgKey.FLOW_CONFIGURATION_ROOT_ELEMENT_NOT_DIAGNOSTIC, node.getLocalName()));
        }
        logger.internal("Defining Diagnostic task list");
        Node tasksNode = null;
        Node abortsNode = null;
        for (Node child = node.getFirstChild(); child != null; child = child.getNextSibling()) {
            if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, REGISTER_ELEMENT)) {
                loadRegister(child);
            }            
            else if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, TASKS_ELEMENT)) {
                tasksNode = child;
            }
            else if (DOMUtils.isElement(child, WORKFLOW_SCHEMA_NAMESPACE, ABORT_TASKS_ELEMENT)) {
                abortsNode = child;
            }
        }
        for (RegisterTemplate r : this.registerVars.values()) {
            if (r.getInputName() != null) {
                throw new FlowException(logger.error(MsgKey.FLOW_DIAGNOSTIC_INPUTS_NOT_ALLOWED));
            }
        }
        this.registerVars.put(FAILED_OPERATION_ID, REGISTER_OPERATION_ID);
        this.registerVars.put(FAILED_OPERATION_CLIENT_REQUEST_ID, REGISTER_OPERATION_CLIENT_REQUEST_ID);
        this.registerVars.put(FAILED_OPERATION_TYPE, REGISTER_OPERATION_TYPE);
        this.registerVars.put(FAILED_OPERATION_REGISTERS, REGISTER_OPERATION_REGISTERS);

        this.taskSteps = loadSteps(tasksNode);
        this.abortSteps = loadSteps(abortsNode);
        validate();
    }
}
