/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/tasks/diag/DefaultDiagnosticSetup.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.tasks.diag;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import com.netapp.common.MsgKey;
import com.netapp.common.flow.Task;
import com.netapp.common.flow.TaskInput;
import com.netapp.common.flow.TaskOutput;
import com.netapp.common.logging.NALogger;

/**
 * Default setup task for diagnostics.
 * This creates a scratch directory for later use and initializes
 * a list for additional files to include in the output.
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DefaultDiagnosticSetup implements Task {
    private static final NALogger logger = NALogger.getLogger(DefaultDiagnosticSetup.class);
  
    private String failedOperationId;
    private String failedOperationType;
    
    private String scratchPath;
    private Set<String> additionalFiles = new HashSet<String>();
    /**
     * @see com.netapp.common.flow.Task#execute()
     */
    public void execute() throws Exception {
        assert this.failedOperationId != null;
        assert this.failedOperationType != null;
        
        String basePath = System.getProperty("java.io.tmpdir") + File.separator + failedOperationType + "_" + failedOperationId + "_diag";
        
        int suffix = 1;
        File path = new File(basePath);
        while (path.exists()) {
            path = new File(basePath + "_" + suffix);
        }
        logger.info(MsgKey.FLOW_TASK_DIAG_SCRATCH_DIRECTORY, path);
        path.mkdir();
        this.scratchPath = path.getAbsolutePath();
    }

    @TaskInput
    public void setFailedOperationId(String failedOperationId) {
        this.failedOperationId = failedOperationId;
    }
    
    @TaskInput
    public void setFailedOperationType(String failedOperationType) {
        this.failedOperationType = failedOperationType;
    }
    
    @TaskOutput
    public String getScratchPath() {
        return this.scratchPath;
    }
    
    @TaskOutput
    public Set<String> getAddtionalFiles() {
        return this.additionalFiles;
    }
}
