/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/aix/DFResultConstructor.java#1 $
 *
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.aix;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.unix.DFResult;
import com.netapp.common.platform.unix.DFResult.FileInfo;

/**
 * Parses the result of the 'df' command on AIX.
 * 
 * TODO: AIX's df doesn't (seem) to support a method of getting the mount
 * type in the output like Linux does with -T. Right now, we assume
 * anything with a block device of filer:/export is nfs, and everything
 * else is unknown. We should try to correlate the output of df against 
 * mount. -jrs
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DFResultConstructor extends com.netapp.common.platform.unix.DFResultConstructor {
    private static final NALogger logger = NALogger.getLogger(DFResultConstructor.class);

    /**
     * Matches 'df -P' output on AIX:
     * "triggerfish:/vol/fg_astoria_9iR2_data    8388608   2554696   5833912      31% /mnt/triggerfish/fg_astoria_9iR2_data"
     * Collects the device (1st) and mount point (6th) entries.
     */
    private static final Pattern PATTERN = Pattern.compile("(\\S+)\\s+\\S+\\s+\\S+\\s+\\S+\\s+\\S+\\s+(\\S+)\\s*");

    DFResultConstructor() {
        super(1, 1);
    }
    
    /**
     * Parse the output from 'df' into a result set.
     * Note, we don't have full support for filesystem types on AIX yet, as
     * the stock userland df doesn't make the mount type available. Anything
     * that looks like an nfs mount is assumed to be nfs on WAFL, and anything
     * else is unknown for the moment.
     */
    @Override
    protected DFResult doConstruct(List<File> fileList, ShellResult shellResult) {
        DFResult dfResult = new DFResult();

        int outputLineIndex = 1;
        for (File file : fileList) {
            String dfLine = shellResult.getStdOut().get(outputLineIndex++);

            Matcher matcher = PATTERN.matcher(dfLine);

            if (!matcher.matches()) {
                String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut().toString());
                throw new PlatformException(error);
            }

            String fileSystem = matcher.group(1);
            String mountPoint = matcher.group(2);

            String filer = null;
            String export = null;

            if (fileSystem.contains(":")) {
                String[] split = fileSystem.split(":");
                if (split.length != 2) {
                    String error = logger.error(MsgKey.PLATFORM_ERROR_PARSING_D_F_OUTPUT, shellResult.getStdOut()
                        .toString());
                    throw new PlatformException(error);
                }
                filer = split[0];
                export = split[1];
            }

            dfResult.addFileInfo(file, new FileInfo(fileSystem, new File(mountPoint), filer, export));
        }
        return dfResult;
    }
}
