/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/server/ServerMain.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.server;

import java.lang.reflect.InvocationTargetException;

import com.netapp.common.MsgKey;
import com.netapp.common.cli.CliParser;
import com.netapp.common.cli.binding.CommandBinding;
import com.netapp.common.logging.NALogger;

/**
 * Defines a standard server call. The server allows for start/stop commands. This
 * same stub class can be reused to parse just about any CLI by extending this class
 * and setting new default values in the constructor and overriding the getMainClass
 * method.
 * 
 * @author joshb
 *
 */
public class ServerMain {
    private static final NALogger LOGGER = NALogger.getLogger(ServerMain.class);
    
    public ServerMain() {
    }

    public Object getFacade() {
        return new ServerFacade();
    }

    /**
     * Given a facade facade class, run the CLI parser.
     */
    public void execute(Object facade, String [] args) {
        CliParser parser = new CliParser(facade.getClass());

        try {
            CommandBinding binding = parser.parse(args);
            binding.execute(facade);
        }
        catch (InvocationTargetException t) {
            // We handle this one seprately since we're actually interested in
            // the chained exception.
            Throwable e = t.getCause();
            System.out.println(LOGGER.error(MsgKey.CLI_MAIN_FAILED_WITH_EXCEPTION, e, e.getMessage()));
            System.exit(1);
        }
        // CHECKSTYLE:OFF
        catch (Throwable e) {
        // CHECKSTYLE:ON
            // Any other throwable we want to log and die
            // Some other daemon threads have been spawned by now, so we need the explicit die
            System.out.println(LOGGER.error(MsgKey.CLI_MAIN_FAILED_WITH_EXCEPTION, e, e.getMessage()));
            System.exit(1);
        }       
    }

    /**
     * @param args
     */
    public static void main(String[] args) {

        ServerMain main = new ServerMain();
        main.execute(main.getFacade(), args);
    }    
}
