/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/text/VariableEngine.java#1 $
 *
 * Copyright (c) 2006 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.text;

import java.io.StringReader;

import org.apache.commons.jexl.Expression;
import org.apache.commons.jexl.ExpressionFactory;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.text.parser.ASTExpression;
import com.netapp.common.text.parser.ParseException;
import com.netapp.common.text.parser.SimpleNode;
import com.netapp.common.text.parser.VariableExpressionParser;

/**
 * Engine for doing procedural variable substitution of the form
 * ${VARIABLE} or ${VARIABLE.ARGUMENT}
 *
 * @version $Revision: #1 $
 * @author sunday
 */
public class VariableEngine {
    private static final NALogger logger = NALogger.getLogger(VariableEngine.class);
    
    private VariableEngine() {
        
    }
    
    /**
     * Convert input string to output, replacing all variables in the process.
     *
     * @param context The variable context to use when evaluating the input string
     * @param input The input string
     * @return A mapped output string
     */
    public static String replace(VariableContext context, String input) {
        VariableExpressionParser parser = new VariableExpressionParser(new StringReader(input));
        try {
            SimpleNode rootNode = parser.ExpressionScript();
            return VariableEngine.convertNodeToString(context, rootNode);            
        }
        catch (ParseException e) {
            return input;
        }        
    }

    private static String convertNodeToString(VariableContext context, SimpleNode node) {
        StringBuffer childText = new StringBuffer();
        for (int i = 0; i < node.jjtGetNumChildren(); i++) {
            childText.append(VariableEngine.convertNodeToString(context, (SimpleNode)node.jjtGetChild(i)));
        }            
    
        if (node instanceof ASTExpression) {
            return String.valueOf(VariableEngine.evaluateExpression(context, childText.toString()));
        } 
        
        return node.getValue() + childText.toString();
    }
    
    /**
     * Evaluates the input expression and returns the result of the expression.
     * 
     * @param context The variable context to use when evaluating the expression 
     * @param expression The expression to evaluate
     * @return The result of the evaluated expression or null if there was an error
     */
    public static Object evaluateExpression(VariableContext context, String expression) {
        try {
            Expression e = ExpressionFactory.createExpression(expression);
            
            return e.evaluate(context);
        } 
        //CHECKSTYLE:OFF
        //Jexl throws Exception
        catch (Exception e) {
        //CHECKSTYLE:ON
            logger.error(MsgKey.VARIABLE_ENGINE_JEXL_EXCEPTION, e, expression, e.getMessage());
        }
        
        return null;
    }
}

