/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/GUID.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc. 
 * All rights reserved.
 */
package com.netapp.common.util;

import java.io.Serializable;
import java.rmi.dgc.VMID;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Object to create a unique identifier. We wrap VMID so toString
 * looks better. Ripped from IdGenerator in SMO.
 *
 * @author gaddy, sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class GUID implements Serializable, Comparable {
    private static final String VALID_ID_REGEX = "[0-9a-f]{32}";
    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    
    private String value;

    /**
     * Create a new GUID
     */
    public GUID() {
        value = generateHexId();
    }

    /**
     * 
     * @param guid
     * @exception IllegalArgumentException
     */
    public GUID(String guid) {
        this.value = guid.toLowerCase();
        if (!isId(this.value)) {
            throw new IllegalArgumentException("GUID must only contain hexadecimal characters");
        }
    }

    public static boolean isId(String id) {
        return (id != null) && (id.toLowerCase().matches(VALID_ID_REGEX));
    }
    
    /**
     * Compare this GUID to another and return true if they are the same
     */
    @Override
    public boolean equals(Object o) {
        if (o instanceof GUID) {
            return this.value.equals(((GUID)o).value);
        }
        return false;
    }

    /**
     * Compute the hash code for this GUID.
     */
    @Override
    public int hashCode() {
        return value.hashCode();
    }

    /**
     * Return string representation of this GUID
     */
    @Override
    public String toString() {
        return value;
    }

    private static String generateHexId() {
        try {
            String id = new VMID().toString();
            MessageDigest md5Digest = MessageDigest.getInstance("MD5");
            byte[] clientIdHash = md5Digest.digest(id.getBytes());
            return byteArrayToHexString(clientIdHash);
        }
        catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("Could not create ID", e);
        }
        
    }
    
    protected static String byteArrayToHexString(byte[] in) {
        byte ch = 0x00;
        int i = 0; 

        if (in == null || in.length <= 0) {
            return null;
        }

        String[] pseudo = {
            "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", 
            "a", "b", "c", "d", "e", "f"};

        StringBuffer out = new StringBuffer(in.length * 2);

        while (i < in.length) {
            ch = (byte) (in[i] & 0xF0); // Strip off high nibble
            ch = (byte) (ch >>> 4); // shift the bits down
            ch = (byte) (ch & 0x0F);   //     must do this is high order bit is on!
            out.append(pseudo[ch]); // convert the nibble to a String Character
            ch = (byte) (in[i] & 0x0F); // Strip off low nibble 
            out.append(pseudo[ch]); // convert the nibble to a String Character
            i++;
        }
        return new String(out);
    }

    public int compareTo(Object o) {
        if (o instanceof GUID) {
            return this.value.compareTo(((GUID)o).value);
        }
        else if (o instanceof String) {
            return this.value.compareTo((String)o);
        }
        throw new ClassCastException("Cannot compare type " + o.getClass() + " to " + this.getClass());
    }
}
