/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/RegexFileFilter.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.io.File;
import java.io.FileFilter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RegexFileFilter implements FileFilter {
    private String regex;
    private FileType fileType;
    private Pattern regexPattern;
    
    public enum FileType {
        FILE,
        DIRECTORY,
        ALL;
    }
    
    public RegexFileFilter() {
        this.fileType = FileType.ALL;
    }
    
    public RegexFileFilter(String regex, FileType fileType) {
        this.regex = regex;
        this.fileType = fileType;
        this.regexPattern = Pattern.compile(this.regex);
    }
    
    public void setRegex(String regex) {
        this.regex = regex;
        this.regexPattern = Pattern.compile(this.regex);
    }
    
    public boolean accept(File file) {
        return matchesName(file.getName()) && matchesType(file);
    }
    
    private boolean matchesName(String fileName) {
        Matcher matcher = this.regexPattern.matcher(fileName);
        
        return matcher.matches();
    }

    private boolean matchesType(File file) {
        if (FileType.ALL.equals(this.fileType)) {
            return true;
        }
        
        if (FileType.FILE.equals(this.fileType)) {
            return file.isFile();
        }
        
        return file.isDirectory();
    }
}
