/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/parser/Tag.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.parser;

/**
 * Represents a parsed javadoc tag and it's contents
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class Tag {
    /** Type of tag */
    private TagEnum type;
    
    /** Value of this tag */
    private StringBuilder value;
    
    /**
     * Create a new tag of the specified type
     * @param type Type of tag
     * @param contents Initial tag contents
     */
    Tag(String type, String contents) {
        this.type = TagEnum.valueOf(type.toUpperCase());
        this.value = new StringBuilder(contents.trim());
    }
    
    /**
     * Append additional contents 
     * @param contents
     */
    void append(String contents) {
        this.value.append(" ").append(contents.trim());
    }
    
    /**
     * Get the type of this tag
     * @return zapidoc type
     */
    TagEnum getType() {
        return this.type;
    }
    
    /**
     * Get the contents of this tag
     * @return tag content
     */
    String getValue() {
        return this.value.toString();
    }
    
    /**
     * Get the first word in the contents of this tag.
     * 
     * @return first word of contents
     */
    String getFirstWord() {
        return this.getValue().replaceAll("^(\\S+)\\s.*$", "$1");
    }
    
    /**
     * Check if this tag is of the supplied type
     * @param type Type to compare against
     * @return true if this tag is of type, false otherwise
     */
    boolean is(TagEnum type) {
        return this.type.equals(type);
    }
    
    /**
     * Get a string representation of this tag
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "@" + this.type + " " + this.value;
    }
}